searchData={"items":[{"type":"module","title":"Retry","doc":"Provides a convenient interface to retrying behavior. All durations are\nspecified in milliseconds.\n\nExamples\n\n    use Retry\n    import Stream\n\n    retry with: exponential_backoff |> randomize |> cap(1_000) |> expiry(10_000) do\n    # interact with external service\n    end\n\n    retry with: linear_backoff(10, 2) |> cap(1_000) |> take(10) do\n    # interact with external service\n    end\n\n    retry with: cycle([500]) |> take(10) do\n    # interact with external service\n    end\n\nThe first retry will exponentially increase the delay, fudging each delay up\nto 10%, until the delay reaches 1 second and then give up after 10 seconds.\n\nThe second retry will linearly increase the retry by a factor of 2 from 10ms giving up after 10 attempts.\n\nThe third example shows how we can produce a delay stream using standard\n`Stream` functionality. Any stream of integers may be used as the value of\n`with:`.","ref":"Retry.html"},{"type":"macro","title":"Retry.retry/2","doc":"Retry a block of code delaying between each attempt the duration specified by\nthe next item in the `with` delay stream.\n\nBoth the values and exceptions that will be retried can be customized. To control which values\nwill be retried, provide the `atoms` option. To control which exceptions are retried, provide\nthe `rescue_only` option. For example:\n\n```\nretry with: ..., atoms: [:not_ok], rescue_only: [CustomError] do\n  ...\nend\n```\n\nBoth `atoms` and `rescue_only` can accept a number of different types:\n\n* An atom (for example: `:not_okay`, `SomeStruct`, or `CustomError`). In this case, the `do`\n  block will be retried in any of the following cases:\n  * The atom itself is returned\n  * The atom is returned in the first position of a two-tuple (for example, `{:not_okay, _}`)\n  * A struct of that type is returned/raised\n* The special atom `:all`. In this case, all values/exceptions will be retried.\n* A function (for example: `fn val -> String.starts_with?(val, \"ok\") end`) or partial function\n  (for example: `fn {:error, %SomeStruct{reason: \"busy\"}} -> true`). The function will be called\n  with the return value and the `do` block will be retried if the function returns a truthy value.\n  If the function returns a falsy value or if no function clause matches, the `do` block\n  will not be retried.\n* A list of any of the above. The `do` block will be retried if any of the items in the list\n  matches.\n\nThe `after` block evaluates only when the `do` block returns a valid value before timeout.\n\nOn the other hand, the `else` block evaluates only when the `do` block remains erroneous after timeout.\n\nExample\n\n    use Retry\n\n    retry with: exponential_backoff() |> cap(1_000) |> expiry(1_000), rescue_only: [CustomError] do\n      # interact with external service\n    after\n      result -> result\n    else\n      error -> error\n    end\n\nThe `after` and `else` clauses are optional. By default, a successful value is just returned. If\nthe timeout expires, the last erroneous value is returned or the last exception is re-raised.\nEssentially, this:\n\n    retry with: ... do\n      ...\n    end\n\nIs equivalent to:\n\n    retry with: ... do\n      ...\n    after\n      result -> result\n    else\n      e when is_exception(e) -> raise e\n      e -> e\n    end","ref":"Retry.html#retry/2"},{"type":"macro","title":"Retry.retry_while/2","doc":"Retry a block of code until `halt` is emitted delaying between each attempt\nthe duration specified by the next item in the `with` delay stream.\n\nThe return value for `block` is expected to be `{:cont, result}`, return\n`{:halt, result}` to end the retry early.\n\nAn accumulator can also be specified which might be handy if subsequent\nretries are dependent on the previous ones.\n\nThe initial value of the accumulator is given as a keyword argument `acc:`.\nWhen the `:acc` key is given, its value is used as the initial accumulator\nand the `do` block must be changed to use `->` clauses, where the left side\nof `->` receives the accumulated value of the previous iteration and\nthe expression on the right side must return the `:cont`/`:halt` tuple\nwith new accumulator value as the second element.\n\nOnce `:halt` is returned from the block, or there are no more elements,\nthe accumulated value is returned.\n\nExample\n\n    retry_while with: linear_backoff(500, 1) |> take(5) do\n      call_service\n      |> case do\n        result = %{\"errors\" => true} -> {:cont, result}\n        result -> {:halt, result}\n      end\n    end\n\nExample with `acc:`\n\n    retry_while acc: 0, with: linear_backoff(500, 1) |> take(5) do\n      acc ->\n        call_service\n        |> case do\n          %{\"errors\" => true} -> {:cont, acc + 1}\n          result -> {:halt, result}\n        end\n    end","ref":"Retry.html#retry_while/2"},{"type":"macro","title":"Retry.wait/2","doc":"Wait for a block of code to be truthy delaying between each attempt\nthe duration specified by the next item in the delay stream.\n\nThe `after` block evaluates only when the `do` block returns a truthy\nvalue. On the other hand, the `else` block evaluates only when the\n`do` block remains falsy after timeout.Both are optional. By default,\na success value will be returned as `{:ok, value}` and an erroneous\nvalue will be returned as `{:error, value}`.\n\nExample\n\n    wait linear_backoff(500, 1) |> take(5) do\n      we_there_yet?\n    after\n      _ ->\n        {:ok, \"We have arrived!\"}\n    else\n      _ ->\n        {:error, \"We're still on our way :(\"}\n    end","ref":"Retry.html#wait/2"},{"type":"module","title":"Retry.Annotation","doc":"A @retry annotation that will retry the function according to the retry settings.\n\nExamples\n\n    use Retry.Annotation\n\n    @retry with: constant_backoff(100) |> Stream.take(10)\n    def some_func(arg) do\n      # ...\n    end","ref":"Retry.Annotation.html"},{"type":"macro","title":"Retry.Annotation.before_compile/1","doc":"","ref":"Retry.Annotation.html#before_compile/1"},{"type":"function","title":"Retry.Annotation.on_definition/6","doc":"","ref":"Retry.Annotation.html#on_definition/6"},{"type":"module","title":"Retry.DelayStreams","doc":"This module provide a set of helper functions that produce delay streams for\nuse with `retry`.","ref":"Retry.DelayStreams.html"},{"type":"function","title":"Retry.DelayStreams.cap/2","doc":"Returns a stream that is the same as `delays` except that the delays never\nexceed `max`. This allow capping the delay between attempts to some max value.\n\nExample\n\n    retry with: exponential_backoff() |> cap(10_000) do\n      # ...\n    end\n\nProduces an exponentially increasing delay stream until the delay reaches 10\nseconds at which point it stops increasing","ref":"Retry.DelayStreams.html#cap/2"},{"type":"function","title":"Retry.DelayStreams.constant_backoff/1","doc":"Returns a constant stream of delays.\n\nExample\n\n    retry with: constant_backoff(50) do\n      # ...\n    end","ref":"Retry.DelayStreams.html#constant_backoff/1"},{"type":"function","title":"Retry.DelayStreams.expiry/3","doc":"Returns a delay stream that is the same as `delays` except it limits the total\nlife span of the stream to `time_budget`. This calculation takes the execution\ntime of the block being retried into account.\n\nThe execution of the code within the block will not be interrupted, so\nthe total time of execution may run over the `time_budget` depending on how\nlong a single try will take.\n\nOptionally, you can specify a minimum delay so the smallest value doesn't go\nbelow the threshold.\n\nExample\n\n    retry with: exponential_backoff() |> expiry(1_000) do\n      # ...\n    end\n\nProduces a delay stream that ends after 1 second has elapsed since its\ncreation.","ref":"Retry.DelayStreams.html#expiry/3"},{"type":"function","title":"Retry.DelayStreams.exponential_backoff/2","doc":"Returns a stream of delays that increase exponentially.\n\nExample\n\n    retry with: exponential_backoff do\n      # ...\n    end","ref":"Retry.DelayStreams.html#exponential_backoff/2"},{"type":"type","title":"Retry.DelayStreams.factor/0","doc":"","ref":"Retry.DelayStreams.html#t:factor/0"},{"type":"function","title":"Retry.DelayStreams.jitter/1","doc":"Returns a stream in which each element of `delays` is randomly adjusted to a number\nbetween 1 and the original delay.\n\nExample\n\n    retry with: exponential_backoff() |> jitter() do\n      # ...\n    end","ref":"Retry.DelayStreams.html#jitter/1"},{"type":"function","title":"Retry.DelayStreams.linear_backoff/2","doc":"Returns a stream of delays that increase linearly.\n\nExample\n\n    retry with: linear_backoff(50, 2) do\n      # ...\n    end","ref":"Retry.DelayStreams.html#linear_backoff/2"},{"type":"function","title":"Retry.DelayStreams.randomize/2","doc":"Returns a stream in which each element of `delays` is randomly adjusted no\nmore than `proportion` of the delay.\n\nExample\n\n    retry with: exponential_backoff() |> randomize do\n      # ...\n    end\n\nProduces an exponentially increasing delay stream where each delay is randomly\nadjusted to be within 10 percent of the original value","ref":"Retry.DelayStreams.html#randomize/2"},{"type":"extras","title":"README","doc":"[![Build Status](https://github.com/safwank/ElixirRetry/actions/workflows/main.yml/badge.svg)](https://github.com/safwank/ElixirRetry/actions/workflows/main.yml)\n\n# ElixirRetry\n\nSimple Elixir macros for linear retry, exponential backoff and wait with composable delays.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"Add `retry` to your list of dependencies in `mix.exs`:\n\n```elixir\n  def deps do\n    [{:retry, \"~> 0.18\"}]\n  end\n```\n\nEnsure `retry` is started before your application:\n\n```elixir\n  def application do\n    [applications: [:retry]]\n  end\n```","ref":"readme.html#installation"},{"type":"extras","title":"Documentation - README","doc":"Check out the [API reference](https://hexdocs.pm/retry/api-reference.html) for the latest documentation.","ref":"readme.html#documentation"},{"type":"extras","title":"Features - README","doc":"","ref":"readme.html#features"},{"type":"extras","title":"Retrying - README","doc":"The `retry([with: _,] do: _, after: _, else: _)` macro provides a way to retry a block of code on failure with a variety of delay and give up behaviors. By default, the execution of a block is considered a failure if it returns `:error`, `{:error, _}` or raises a runtime error.\n\nBoth the values and exceptions that will be retried can be customized. To control which values will be retried, provide the `atoms` option. To control which exceptions are retried, provide the `rescue_only` option. For example:\n\n```\nretry with: ..., atoms: [:not_ok], rescue_only: [CustomError] do\n  ...\nend\n```\n\nBoth `atoms` and `rescue_only` can accept a number of different types:\n\n* An atom (for example: `:not_okay`, `SomeStruct`, or `CustomError`). In this case, the `do` block will be retried in any of the following cases:\n  * The atom itself is returned\n  * The atom is returned in the first position of a two-tuple (for example, `{:not_okay, _}`)\n  * A struct of that type is returned/raised\n* The special atom `:all`. In this case, all values/exceptions will be retried.\n* A function (for example: `fn val -> String.starts_with?(val, \"ok\") end`) or partial function (for example: `fn {:error, %SomeStruct{reason: \"busy\"}} -> true`). The function will be called with the return value and the `do` block will be retried if the function returns a truthy value. If the function returns a falsy value or if no function clause matches, the `do` block will not be retried.\n* A list of any of the above. The `do` block will be retried if any of the items in the list matches.\n\nThe `after` block evaluates only when the `do` block returns a valid value before timeout. On the other hand, the `else` block evaluates only when the `do` block remains erroneous after timeout. Both are optional. By default, the `else` clause will return the last erroneous value or re-raise the last exception. The default `after` clause will simply return the last successful value.\n\n#### Example -- constant backoff\n\n```elixir\nresult = retry with: constant_backoff(100) |> Stream.take(10) do\n  ExternalApi.do_something # fails if other system is down\nafter\n  result -> result\nelse\n  error -> error\nend\n```\n\nThis example retries every 100 milliseconds and gives up after 10 attempts.\n\n#### Example -- linear backoff\n\n```elixir\nresult = retry with: linear_backoff(10, 2) |> cap(1_000) |> Stream.take(10) do\n  ExternalApi.do_something # fails if other system is down\nafter\n  result -> result\nelse\n  error -> error\nend\n```\n\nThis example increases the delay linearly with each retry, starting with 10 milliseconds, caps the delay at 1 second and gives up after 10 attempts.\n\n#### Example -- exponential backoff\n\n```elixir\nresult = retry with: exponential_backoff() |> randomize |> expiry(10_000), rescue_only: [TimeoutError] do\n  ExternalApi.do_something # fails if other system is down\nafter\n  result -> result\nelse\n  error -> error\nend\n```\n\n#### Example -- optional clauses\n\n```elixir\nresult = retry with: constant_backoff(100) |> Stream.take(10) do\n  ExternalApi.do_something # fails if other system is down\nend\n```\n\nThis example is equivalent to:\n\n```elixir\nresult = retry with: constant_backoff(100) |> Stream.take(10) do\n  ExternalApi.do_something # fails if other system is down\nafter\n  result -> result\nelse\n  e when is_exception(e) -> raise e\n  e -> e\nend\n```\n\n#### Example -- retry annotation\n\n```elixir\nuse Retry.Annotation\n\n@retry with: constant_backoff(100) |> Stream.take(10)\ndef some_func(arg) do\n  ExternalApi.do_something # fails if other system is down\nend\n```\n\nThis example shows how you can annotate a function to retry every 100 milliseconds and gives up after 10 attempts.\n\n#### Delay streams\n\nThe `with:` option of `retry` accepts any `Stream` that yields integers. These integers will be interpreted as the amount of time to delay before retrying a failed operation. When the stream is exhausted `retry` will give up, returning the last value of the block.\n\n##### Example\n\n```elixir\nresult = retry with: Stream.cycle([500]) do\n  ExternalApi.do_something # fails if other system is down\nafter\n  result -> result\nelse\n  error -> error  \nend\n```\n\nThis will retry failures forever, waiting 0.5 seconds between attempts.\n\n`Retry.DelayStreams` provides a set of fully composable helper functions for building useful delay behaviors such as the ones in previous examples. See the `Retry.DelayStreams` module docs for full details and addition behavior not covered here. For convenience these functions are imported by `use Retry` so you can, usually, use them without prefixing them with the module name.","ref":"readme.html#retrying"},{"type":"extras","title":"Waiting - README","doc":"Similar to `retry(with: _, do: _)`, the `wait(delay_stream, do: _, after: _, else: _)` macro provides a way to wait for a block of code to be truthy with a variety of delay and give up behaviors. The execution of a block is considered a failure if it returns `false` or `nil`.\n\n```elixir\nwait constant_backoff(100) |> expiry(1_000) do\n  we_there_yet?\nafter\n  _ ->\n    {:ok, \"We have arrived!\"}\nelse\n  _ ->\n    {:error, \"We're still on our way :(\"}\nend\n```\n\nThis example retries every 100 milliseconds and expires after 1 second.\n\nThe `after` block evaluates only when the `do` block returns a truthy value. On the other hand, the `else` block evaluates only when the `do` block remains falsy after timeout. Both are optional. By default, a success value will be returned as `{:ok, value}` and an erroneous value will be returned as `{:error, value}`.\n\nPretty nifty for those pesky asynchronous tests and building more reliable systems in general!","ref":"readme.html#waiting"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}